package models

import (
	"errors"
	"fmt"
	"log/slog"
)

import (
	"gorm.io/driver/postgres"
	"gorm.io/gorm"
)

import (
	"resource-server/common"
	"resource-server/logger"
)

var (
	DB  *gorm.DB
	err error
)

func panicHelper(err error) {
	if err != nil {
		panic(err)
	}
}

func migrate() {
	panicHelper(DB.AutoMigrate(&User{}))
	var user User
	err := DB.First(&user).Error
	if errors.Is(err, gorm.ErrRecordNotFound) {
		newAdmin := User{Name: "admin"}
		newAdmin.GenerateFromPassword([]byte("123456"))
		DB.Create(&newAdmin)
	}

	panicHelper(DB.AutoMigrate(&ConfigItem{}))
	var count int64
	DB.Model(&ConfigItem{}).Count(&count)
	if count == 0 {
		configItem := ConfigItem{Key: "TimeFormat", Value: "HH:mm:ss", OptionKey: "TimeFormat", ValueType: "string", Description: "", Order: 1, IsEnabled: true, Uri: "System/TimeFormat", DefaultValue: ""}
		DB.Create(&configItem)
		configItem = ConfigItem{Key: "DateFormat", Value: "MM/dd/yyyy", OptionKey: "DateFormat", ValueType: "string", Description: "", Order: 1, IsEnabled: true, Uri: "System/DateFormat", DefaultValue: ""}
		DB.Create(&configItem)
		configItem = ConfigItem{Key: "DoseUnit", Value: "mGy", OptionKey: "DoseUnit", ValueType: "string", Description: "", Order: 1, IsEnabled: true, Uri: "System/DoseUnit", DefaultValue: ""}
		DB.Create(&configItem)
		configItem = ConfigItem{Key: "WeightUnit", Value: "kg", OptionKey: "WeightUnit", ValueType: "string", Description: "", Order: 1, IsEnabled: true, Uri: "System/WeightUnit", DefaultValue: ""}
		DB.Create(&configItem)
		configItem = ConfigItem{Key: "LengthUnit", Value: "cm", OptionKey: "LengthUnit", ValueType: "string", Description: "", Order: 1, IsEnabled: true, Uri: "System/LengthUnit", DefaultValue: ""}
		DB.Create(&configItem)
		configItem = ConfigItem{Key: "DefaultImageFontSize", Value: "100", OptionKey: "DefaultFontSize", ValueType: "int", Description: "", Order: 1, IsEnabled: true, Uri: "ImageProcess/DefaultImageFontSize", DefaultValue: ""}
		DB.Create(&configItem)
		configItem = ConfigItem{Key: "DefaultImageFontFamily", Value: "Arial Unicode MS", OptionKey: "DefaultFontFamily", ValueType: "string", Description: "", Order: 1, IsEnabled: true, Uri: "ImageProcess/DefaultImageFontFamily", DefaultValue: ""}
		DB.Create(&configItem)
		configItem = ConfigItem{Key: "TagFontSize", Value: "100", OptionKey: "FontSize", ValueType: "int", Description: "", Order: 1, IsEnabled: true, Uri: "ImageProcess/TagFontSize", DefaultValue: ""}
		DB.Create(&configItem)
		configItem = ConfigItem{Key: "TagFontFamily", Value: "Arial", OptionKey: "FontFamily", ValueType: "string", Description: "", Order: 1, IsEnabled: true, Uri: "ImageProcess/TagFontFamily", DefaultValue: ""}
		DB.Create(&configItem)
		configItem = ConfigItem{Key: "HidePatientNameDelimiters", Value: "1", OptionKey: "", ValueType: "bool", Description: "", Order: 1, IsEnabled: true, Uri: "Patient/HidePatientNameDelimiters", DefaultValue: ""}
		DB.Create(&configItem)
		configItem = ConfigItem{Key: "FirstNameFirstDisplay", Value: "1", OptionKey: "", ValueType: "bool", Description: "", Order: 1, IsEnabled: true, Uri: "Patient/FirstNameFirstDisplay", DefaultValue: ""}
		DB.Create(&configItem)
		configItem = ConfigItem{Key: "LRDisplayMode", Value: "L/R", OptionKey: "LRDisplayMode", ValueType: "string", Description: "", Order: 1, IsEnabled: true, Uri: "ImageProcess/LRDisplayMode", DefaultValue: ""}
		DB.Create(&configItem)
		configItem = ConfigItem{Key: "PrintTagFontLevel", Value: "6", OptionKey: "", ValueType: "int", Description: "", Order: 1, IsEnabled: true, Uri: "ImageProcess/PrintTagFontLevel", DefaultValue: ""}
		DB.Create(&configItem)
		configItem = ConfigItem{Key: "PrintTagFontFamily", Value: "Arial", OptionKey: "", ValueType: "string", Description: "", Order: 1, IsEnabled: true, Uri: "ImageProcess/PrintTagFontFamily", DefaultValue: ""}
		DB.Create(&configItem)
	}
}

func SetupGorm(m bool) {
	slog.Info("setup gorm")

	dsn := fmt.Sprintf("host=%s user=%s password=%s dbname=%s port=%d sslmode=disable TimeZone=Asia/Shanghai",
		common.PostgresConfig.Ip,
		common.PostgresConfig.Username,
		common.PostgresConfig.Password,
		common.PostgresConfig.Name,
		common.PostgresConfig.Port,
	)
	if common.LoggerConfig.LogLevel == "debug" {
		DB, err = gorm.Open(postgres.Open(dsn), &gorm.Config{
			Logger: logger.NewGormLogger(logger.WithGroup("gorm")),
		})
	} else {
		DB, err = gorm.Open(postgres.Open(dsn), &gorm.Config{})
	}
	if err != nil {
		panic(err)
	}
	if m {
		migrate()
	}
	slog.Info("setup gorm ok")
}
